/* $XFree86: xc/config/cf/cross.def,v 1.1 2000/12/08 22:09:34 keithp Exp $ */
/*
 * This file contains redefinitions of some symbols to enable
 * cross compilation: e.g. paths for include files and paths to
 * compiler images.  It will have to be edited to reflect these
 * given your local configuration.
 */
#if 0
#undef i386Architecture
#define Arm32Architecture

#undef OptimizedCDebugFlags
#define OptimizedCDebugFlags 	-O2
#define ServerCDebugFlags	-O2
#undef StandardDefines
#define StandardDefines 	-Dlinux -D__arm__ -D_POSIX_SOURCE \
				-D_BSD_SOURCE -D_GNU_SOURCE -DX_LOCALE
#undef CcCmd
#define StdIncDir /skiff/local/arm-linux/include
#define PreIncDir
#undef PostIncDir
#define PostIncDir /skiff/local/lib/gcc-lib/arm-linux/2.95.2/include
#define CcCmd /skiff/local/bin/arm-linux-gcc
#undef CplusplusCmd
#define HasCplusplus YES
#define CplusplusCmd /skiff/local/bin/arm-linux-g++
#define DoRanlibCmd YES
#define RanlibCmd /skiff/local/bin/arm-linux-ranlib
#undef ExtraLoadFlags
#define ExtraLoadFlags
#define FbNoPixelAddrCode
#undef TermcapLibrary
#define TermcapLibrary -ltermcap

#undef LdPostLib
#define LdPostLib	-L/skiff/local/arm-linux/lib

#undef ExtensionOSDefines
#define ExtensionOSDefines

#define ServerXdmcpDefines	/**/

#define HostCcCmd  cc
#endif
#include <cross.rules>
